"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.plugin = void 0;
const graphql_codegen_core_1 = require("@cobbl/graphql-codegen-core");
const change_case_all_1 = require("change-case-all");
const graphql_1 = require("graphql");
const defaultConfig = {
    prefix: 'test',
};
const plugin = (schema, sources, config) => {
    config = { ...defaultConfig, ...config };
    const documentNodes = sources.reduce((acc, document) => {
        if (document.document) {
            acc.push(document.document);
        }
        return acc;
    }, []);
    const documentNode = (0, graphql_1.concatAST)(documentNodes);
    const fragmentMap = documentNode.definitions.reduce((acc, definition) => {
        if (definition.kind === graphql_1.Kind.FRAGMENT_DEFINITION) {
            acc[definition.name.value] = definition;
        }
        return acc;
    }, {});
    const lines = [];
    (0, graphql_1.visit)(documentNode, {
        OperationDefinition: {
            enter(node) {
                if (!node.name) {
                    throw new Error(`All graphql operation definitions should have a unique name: \n ${(0, graphql_1.print)(node)}`);
                }
                const type = node.operation === 'mutation' ? 'Mutation' : 'Query';
                // Mimic the default naming convention.
                // See https://www.graphql-code-generator.com/docs/getting-started/naming-convention#namingconvention
                const name = (0, change_case_all_1.pascalCase)(`${node.name.value}${type}`);
                const operationFragments = (0, graphql_codegen_core_1.getFragmentFromOperation)(node, fragmentMap);
                const query = `${operationFragments.map(graphql_1.print)}${(0, graphql_1.print)(node)}`;
                lines.push(``);
                lines.push(`export const ${name}Source = \``);
                lines.push(`${query}\`;`);
                lines.push(``);
                lines.push(`export function ${config.prefix}${name}(`);
                lines.push(`  graphqlArgs: Partial<GraphQLArgs> & Pick<GraphQLArgs, 'schema'>,`);
                lines.push(`  variables?: ${name}Variables`);
                lines.push(`) {`);
                lines.push(`  return graphql({ ...graphqlArgs, source: ${name}Source, variableValues: variables }) as Promise<ExecutionResult<${name}>>`);
                lines.push(`};`);
            },
        },
    });
    return {
        prepend: [`import { graphql, ExecutionResult, GraphQLArgs } from 'graphql'`],
        content: lines.join('\n'),
    };
};
exports.plugin = plugin;
